/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.callstack;

import java.util.Collection;
import java.util.Collections;

public class ComponentRecord {

	private final String serverNode;
	private final String component;
	private final String application;
	private final String module;
	private String componentClass = null;
	private String stringRepresentation;
	
	public ComponentRecord(String serverNode, String application, String module, String component) {
		this.serverNode = serverNode;
		this.application = application;
		this.module = module;
		this.component = component;
	}

	public static String getComponentIdentifier(String serverNode, String application, String module,
			String component) {
		StringBuilder name = new StringBuilder();
		if (serverNode != null && !serverNode.isEmpty()) {
			name.append(serverNode).append("/");
		}
		if (application != null && !application.isEmpty()) {
			name.append(application).append("/");
		}
		if (module != null && !module.isEmpty()) {
			name.append(module).append("/");
		}
		name.append(component);
		return name.toString();
	}

	public void setComponentClass(String componentClass) {
		this.componentClass = componentClass;
	}
	
	public String getComponentClass() {
		return componentClass;
	}
	
	public String getComponentName() {
		return component;
	}

	public String getServerNode() {
		return serverNode;
	}

	public String getApplication() {
		return application;
	}

	public String getModule() {
		return module;
	}

	public Collection<OperationRecord> getOperations() {
		return Collections.emptyList();
	}
	
	@Override
	public String toString() {
		if (stringRepresentation == null) {
			stringRepresentation = getComponentIdentifier(serverNode, application, module, component);
		}
		return stringRepresentation;
	}

}
