/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.extension;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.jboss.as.controller.AttributeDefinition;
import org.jboss.as.controller.PersistentResourceDefinition;
import org.jboss.as.controller.SimpleAttributeDefinitionBuilder;
import org.jboss.as.controller.registry.AttributeAccess;
import org.jboss.dmr.ModelNode;
import org.jboss.dmr.ModelType;

import tools.descartes.prisma.agent.wildfly.filters.FilterDefinition;
import tools.descartes.prisma.agent.wildfly.parameters.ParameterCharacterizationDefinition;

/**
 * @author <a href="mailto:tcerar@redhat.com">Tomaz Cerar</a>
 */
public class SubsystemDefinition extends PersistentResourceDefinition {
    public static final SubsystemDefinition INSTANCE = new SubsystemDefinition();

	public static final AttributeDefinition REPOSITORY_HOST = new SimpleAttributeDefinitionBuilder(
			SubsystemExtension.REPOSITORY_HOST_ATTRIBUTE, ModelType.STRING).setAllowExpression(true)
					.setXmlName(SubsystemExtension.REPOSITORY_HOST_ATTRIBUTE)
					.setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES).setDefaultValue(new ModelNode("127.0.0.1"))
					.setAllowNull(false).build();

	public static final AttributeDefinition REPOSITORY_USER = new SimpleAttributeDefinitionBuilder(
			SubsystemExtension.REPOSITORY_USER_ATTRIBUTE, ModelType.STRING).setAllowExpression(true)
					.setXmlName(SubsystemExtension.REPOSITORY_USER_ATTRIBUTE)
					.setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES).setDefaultValue(new ModelNode("user"))
					.setAllowNull(false).build();

	public static final AttributeDefinition REPOSITORY_PASSWORD = new SimpleAttributeDefinitionBuilder(
			SubsystemExtension.REPOSITORY_PASSWORD_ATTRIBUTE, ModelType.STRING).setAllowExpression(true)
					.setXmlName(SubsystemExtension.REPOSITORY_PASSWORD_ATTRIBUTE)
					.setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES).setDefaultValue(new ModelNode("password"))
					.setAllowNull(false).build();

	public static final AttributeDefinition SCOPE = new SimpleAttributeDefinitionBuilder(
			SubsystemExtension.SCOPE_ATTRIBUTE, ModelType.STRING).setAllowExpression(true)
					.setXmlName(SubsystemExtension.SCOPE_ATTRIBUTE).setFlags(AttributeAccess.Flag.RESTART_ALL_SERVICES)
					.setDefaultValue(new ModelNode("application/unknown")).setAllowNull(false).build();

    private SubsystemDefinition() {
        super(SubsystemExtension.SUBSYSTEM_PATH,
                SubsystemExtension.getResourceDescriptionResolver(null),
                //We always neeexid to add an 'add' operation
                SubsystemAdd.INSTANCE,
                //Every resource that is added, normally needs a remove operation
                SubsystemRemove.INSTANCE);
    }

	@Override
	public Collection<AttributeDefinition> getAttributes() {
		return Arrays.asList(REPOSITORY_HOST, REPOSITORY_USER, REPOSITORY_PASSWORD, SCOPE);
	}
	
	@Override
	protected List<? extends PersistentResourceDefinition> getChildren() {
		return Arrays.asList(FilterDefinition.INSTANCE, ParameterCharacterizationDefinition.INSTANCE);
	}
}
