/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.filters;

import java.util.Arrays;
import java.util.List;

import org.jboss.as.controller.AbstractAddStepHandler;
import org.jboss.as.controller.AttributeDefinition;
import org.jboss.as.controller.OperationContext;
import org.jboss.as.controller.OperationFailedException;
import org.jboss.as.controller.PathAddress;
import org.jboss.as.controller.ServiceVerificationHandler;
import org.jboss.as.controller.descriptions.ModelDescriptionConstants;
import org.jboss.dmr.ModelNode;
import org.jboss.msc.service.ServiceController;
import org.jboss.msc.service.ServiceTarget;

import tools.descartes.prisma.agent.wildfly.service.FilterService;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

class FilterAdd extends AbstractAddStepHandler {
	
	public static final FilterAdd INSTANCE = new FilterAdd();
	
	private FilterAdd() {
		super(Arrays.asList(FilterDefinition.COMPONENT_PATTERN, FilterDefinition.INTERFACE_PATTERN, FilterDefinition.OPERATION_PATTERN));
	}
	
	@Override
	protected void performRuntime(OperationContext context, ModelNode operation, ModelNode model,
			ServiceVerificationHandler verificationHandler, List<ServiceController<?>> newControllers)
					throws OperationFailedException {
		final PathAddress address = PathAddress.pathAddress(operation.get(ModelDescriptionConstants.OP_ADDR));
		String name = address.getLastElement().getValue();
		
		
		final FilterService service = new FilterService(getResolvedModel(context, model));
		final ServiceTarget target = context.getServiceTarget();
		ServiceController<?> sc = target.addService(MonitoringService.FILTER_NAME.append(name), service)
				.setInitialMode(ServiceController.Mode.ON_DEMAND)
				.install();
		if (newControllers != null) {
			newControllers.add(sc);
		}
	}
	
	private ModelNode getResolvedModel(OperationContext context, ModelNode model) throws OperationFailedException {
        ModelNode resolved = new ModelNode();
        for (AttributeDefinition attribute : attributes) {
            resolved.get(attribute.getName()).set(attribute.resolveModelAttribute(context, model));
        }
        return resolved;
    }

}