/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import java.util.Deque;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.Topic;

import org.jboss.logging.Logger;

import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class MessageInterceptor {
	
	protected static final Logger log = Logger.getLogger(MessageInterceptor.class);

	public static void intercept(Destination destination) {
		MonitoringService monitoringService = MonitoringService.INSTANCE;
		Deque<InvocationRecord> threadCallStack = monitoringService.getCallStack();
		InvocationRecord currentInvocation = threadCallStack.peek();
		if (currentInvocation != null) {
			String destinationName = null;
			String componentClass = null;
			try {
				if (destination instanceof Queue) {
					destinationName = ((Queue)destination).getQueueName();
					componentClass = "javax.jms.Queue";
				} else if (destination instanceof Topic) {
					destinationName = ((Topic)destination).getTopicName();
					componentClass = "javax.jms.Topic";
				}
				if (destinationName != null) {
					ComponentRecord crec = monitoringService.requireComponent("", "", "",
							destinationName);
					if (crec.getComponentClass() == null) {
						crec.setComponentClass(componentClass);
					}
					OperationRecord orec = currentInvocation.requireEndpoint(crec,
							"javax.jms.MessageProducer", "jms", "send");
					InvocationRecord outgoingInvocation = orec.startInvocation();
					outgoingInvocation.finishInvocation(0, 0, 0);

					if (log.isTraceEnabled()) {
						log.trace("Outgoing invocation (" + orec.getInterfaceName() + ", " + orec.getOperationName()
								+ ") -> (" + currentInvocation.getOperation().getInterfaceName() + ", "
								+ currentInvocation.getOperation().getOperationName() + ")");
					}
				}
			} catch (JMSException e) {
				log.error("Error intercepting message.", e);
			}
		} else {
			log.warn("No invocation record found.");
		}
	}
}
