/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.service;

import org.jboss.dmr.ModelNode;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;

import tools.descartes.prisma.agent.wildfly.extension.SubsystemExtension;
import tools.descartes.prisma.agent.wildfly.filters.OperationPredicate;

public class FilterService implements Service<FilterService>  {
	
	private ModelNode definition;
	
	public FilterService(ModelNode definition) {
		this.definition = definition;
	}
	
	@Override
	public FilterService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}
	
	public OperationPredicate getPredicate() {
		ModelNode compPattern = definition.get(SubsystemExtension.COMPONENT_PATTERN);
		ModelNode intPattern = definition.get(SubsystemExtension.INTERFACE_PATTERN);
		ModelNode opPattern = definition.get(SubsystemExtension.OPERATION_PATTERN);
		System.err.println("FilterService: " + compPattern.asString() + " " + intPattern.asString() + " " + opPattern.asString());
		
		return new OperationPredicate(compPattern.asString(), intPattern.asString(), opPattern.asString());
	}

	@Override
	public void start(StartContext arg0) throws StartException {
		System.err.println("FitlerService started: " + definition.toJSONString(true));
	}

	@Override
	public void stop(StopContext arg0) {
		System.err.println("FitlerService stopped");
	}
}
