/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.service;

import org.jboss.as.naming.ImmediateManagedReferenceFactory;
import org.jboss.as.naming.ManagedReferenceFactory;
import org.jboss.as.naming.ServiceBasedNamingStore;
import org.jboss.as.naming.deployment.ContextNames;
import org.jboss.as.naming.service.BinderService;
import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;

import tools.descartes.prisma.agent.wildfly.interceptors.jms.ConnectionFactoryDelegate;

public class InstrumentationService implements Service<InstrumentationService> {
	
	private static final Logger log = Logger.getLogger(InstrumentationService.class);
	
	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "instrumentation");
	
	@Override
	public InstrumentationService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext ctx) throws StartException {
		log.info("Instrument JMS connection factory: java:/JmsXA");
		
//		final ContextNames.BindInfo bindInfo = ContextNames.bindInfoFor("java:jboss/DefaultJMSConnectionFactory");
		final ContextNames.BindInfo bindInfo = ContextNames.bindInfoFor("java:jboss/InterceptedJMSConnectionFactory");
	    final BinderService binderService = new BinderService(bindInfo.getBindName());
	    
	    ConnectionFactoryDelegate delegate = new ConnectionFactoryDelegate("java:/JmsXA");
//	    ConnectionFactoryDelegate delegate = new ConnectionFactoryDelegate("java:jboss/DefaultJMSConnectionFactory");
	     
	    final ManagedReferenceFactory managedReferenceFactory = new ImmediateManagedReferenceFactory(delegate);
	    ctx.getChildTarget()
	             .addService(bindInfo.getBinderServiceName(), binderService)
	             .addInjection(binderService.getManagedObjectInjector(), managedReferenceFactory)
	             .addDependency(bindInfo.getParentContextServiceName(),
	                     ServiceBasedNamingStore.class,
	                     binderService.getNamingStoreInjector())
	             .install();
		
				
		ctx.complete();
	}

	@Override
	public void stop(StopContext ctx) {
		// TODO Auto-generated method stub
		
	}
}
